﻿/**
 *  version:2.1.2
 */
using System.Text;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.Text.RegularExpressions;
using com.xsj.Crasheye.U3D;

public class Crasheye : MonoBehaviour
{
    public string YourAppKeyForAndroid = "YourAppKeyForAndroid";
    public string YourChannelIdForAndroid = "YourChannelIdForAndroid";

    public string YourAppKeyForIOS = "YourAppKeyForIOS";
    public string YourChannelIdForIOS = "YourChannelIdForIOS";

    public static CrasheyeLib crasheyeLib = null;

    void Start()
    {
		#if UNITY_ANDROID
		Crasheye.RegisterLogCallback();
		#endif
		
		#if UNITY_IPHONE
		// ios 初始化还是走以前的流程
		Crasheye.StartInitCrasheye("58e829a0");
		#endif
        DontDestroyOnLoad(gameObject);

        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            SetChannelID(YourChannelIdForAndroid);
            StartInitCrasheye(YourAppKeyForAndroid);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            SetChannelID(YourChannelIdForIOS);
            StartInitCrasheye(YourAppKeyForIOS);
#endif
        }
    }

    /// <summary>
    /// 启动Crasheye
    /// </summary>
    /// <param name="Your_AppKey"></param>
    public static void StartInitCrasheye(string Your_AppKey)
    {        
        RegisterLogCallback();
        
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.Init(Your_AppKey, YourChannelId);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.Init(Your_AppKey, YourChannelId);
#endif
        }
    }

    /// <summary>
    /// 注册脚本捕获
    /// </summary>
    public static void RegisterLogCallback()
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            crasheyeLib = new LibForAndroid();
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            crasheyeLib = new LibForiOS();
#endif
        }

        if (crasheyeLib == null)
        {
            return;
        }

        System.AppDomain.CurrentDomain.UnhandledException += new System.UnhandledExceptionEventHandler(crasheyeLib.OnHandleUnresolvedException);
        SetRegisterLogFunction(crasheyeLib.OnHandleLogCallback);
        Application.RegisterLogCallback(RegisterLogFunction);
    }

    private static void RegisterLogFunction(string logString, string stackTrace, LogType type)
    {
        if (m_RegisterLog != null)
        {
            m_RegisterLog(logString, stackTrace, type);
        }
    }

    public delegate void RegisterLog(string logString, string stackTrace, LogType type);
    private static RegisterLog m_RegisterLog = null;    
    /// <summary>
    /// 脚本异常时回调的函数
    /// </summary>
    /// <param name="registerLogCBFun"></param>
    public static void SetRegisterLogFunction(RegisterLog registerLogCBFun)
    {
        m_RegisterLog += registerLogCBFun;
    }
    
    /// <summary>
    /// 发送脚本异常
    /// </summary>
    /// <param name="ex">Excepiton Info</param>
    public static void SendScriptException(Exception ex)
    {
        string exception = LibCommon.GetJsonStack(ex.Message, ex.StackTrace);

        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            crasheyeLib.SendScriptException(exception);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE            
            crasheyeLib.SendScriptException(exception);
#endif
        }
    }

    private static string YourChannelId = "";
    /// <summary>
    /// 设置渠道号
    /// </summary>
    /// <param name="yourChannelID"></param>
    public static void SetChannelID(string yourChannelID)
    {
        if (yourChannelID.Equals("YourChannelIdForAndroid") || yourChannelID.Equals("YourChannelIdForIOS"))
        {
            return;
        }

        YourChannelId = yourChannelID;
    }

    /// <summary>
    /// 设置是否只在wifi下上报报告文件
    /// </summary>
    /// <param name="enabled"></param>
    public static void SetFlushOnlyOverWiFi(bool enabled)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.SetFlushOnlyOverWiFi(enabled);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.SetFlushOnlyOverWiFi(enabled);
#endif
        }
    }

    /// <summary>
    /// 设置版本号信息
    /// </summary>
    /// <param name="yourAppVersion"></param>
    public static void SetAppVersion(string yourAppVersion)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.SetAppVersion(yourAppVersion);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.SetAppVersion(yourAppVersion);
#endif
        }
        
    }

    /// <summary>
    /// 如果启动了C#堆栈回溯可能会导致某些机型出现宕机
    /// </summary>
    public static void EnableCSharpStackTrace()
    {
#if UNITY_ANDROID
        CrasheyeForAndroid.EnableCSharpStackTrace();
#endif
    }

    /// <summary>
    /// 添加自定义数据
    /// </summary>
    /// <param name="key">Key.</param>
    /// <param name="value">Value.</param>
    public static void AddExtraData(string key, string value)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.AddExtraData(key, value);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.AddExtraData(key, value);
#endif
        }
    }

    /// <summary>
    /// 获取自定义数据
    /// </summary>
    /// <param name="key"></param>
    /// <returns></returns>
    public static string GetExtraData(string key)
    {
        string extraData = "";
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            extraData = CrasheyeForAndroid.GetExtraData(key);
#endif
        }

        return extraData;
    }

    /// <summary>
    /// 移除自定义值
    /// </summary>
    /// <param name="key"></param>
    public static void RemoveExtraData(string key)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.RemoveExtraData(key);
#endif
        }
        else if(Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.RemoveExtraData(key);
#endif
        }
    }

    /// <summary>
    /// 清除自定义数据
    /// </summary>
    public static void CleanExtraData()
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.CleanExtraData();
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.ClearExtraData();
#endif
        }
    }
            
    /// <summary>
    /// 打点数据
    /// </summary>
    /// <param name="breadcrumb">Breadcrumb.</param>
    public static void LeaveBreadcrumb(string breadcrumb)
    {
        if (Application.platform == RuntimePlatform.Android)
        {
#if UNITY_ANDROID
            CrasheyeForAndroid.LeaveBreadcrumb(breadcrumb);
#endif
        }
        else if (Application.platform == RuntimePlatform.IPhonePlayer)
        {
#if UNITY_IPHONE
            CrasheyeForIOS.LeaveBreadcrumb(breadcrumb);
#endif
        }
    }

}